/*
 * C'Gol
 * Project based on Conway's Game of Life
 * 
 * Made by:
 * Alexander Noss
 * Christian Rautenberg
 * David Zentner
 * Philipp Erler
 * Stephan Schneider
 * 
 * As IT project for summer-semester 2011 (Group IT2D)
 * 
 * ####################################################################
 * Field Update
 * Part of C'Gol C-DLL
 * ####################################################################
 * 
 * Last change:     2011 | 07 | 04
 * 
 * Mail me for feedback, version requests etc.: 	ph.erler@gmx.net
 * Look at http://www.hs-esslingen.de/~pherit00/website/index.htm
 * .Net - Framework 4.0 must be installed.
 * 
 */


/*
Informatik Project SS2011
Semester:		IT2D
Author:			Stephan Schneider
*/

#include "fieldUpdate.h"

int extern alive;
int extern dead;
int extern born;
int extern gone;
int **ptrptrTempField;
int **ptrptrFieldToDelete;

int DEBUG = 0;

int i;
int j;
extern int fieldSize;
extern int **ptrptrField;


// get number of living cells
int getAlive()
{
	return alive;
}

// get number of dead cells
int getDead()
{
	return dead;
}

// get number of born cells
int getBorn()
{
	return born;
}

// get number of gone cells
int getGone()
{
	return gone;
}

// creation of local field for next generation
void createTempField()
{

	int i;
	int j;

	ptrptrTempField = (int **)calloc(fieldSize, sizeof(int *));

	for(i = 0; i < fieldSize; ++i)
	{
		ptrptrTempField[i] = (int *)calloc(fieldSize, sizeof(int));
	}

	for(j = 0; j < fieldSize; ++j)
	{
		for(i = 0; i < fieldSize; ++i)
		{
			ptrptrTempField[i][j] = ptrptrField[i][j];
		}
	}

	if (DEBUG) 
	{
		//Test-output for initialisation of new Array
		printf("field is\n");
		for(i = 0; i < fieldSize; ++i)
		{
			for(j = 0; j < fieldSize; ++j)
			{
				printf("%d ", ptrptrTempField[j][i]);
				if (j == fieldSize - 1)
				{
					printf("\n");
				}
			}
		}
	}

}

// check if a selected cell is element of field
int isCellInFieldPlanar(int x, int y)
{
	if (x >= 0 && x < fieldSize)
	{
		if (y >= 0 && y < fieldSize)
		{
			if (DEBUG) printf("valid ");
			return 1;
		}
		else
		{
			if (DEBUG) printf("not y");
		}
	}
	else
	{
		if (y >= 0 && y < fieldSize)
		{
			if (DEBUG) printf("not x");
		}
		else
		{
			if (DEBUG) printf("not xy");
		}
	}
	return 0;
}

// algorithm calculating the next generation
int updateField()
{
	int neighbours = 0;
	alive = 0;
	dead = 0;
	born = 0;
	gone = 0;

	if (fieldSize > 0)
	{
		createTempField();

		if (!DEBUG) printf("created TempField\n");

		for(j = 0; j < fieldSize; ++j)
		{
			neighbours = 0;

			for(i = 0; i < fieldSize; ++i)
			{
				neighbours = 0;

				if (DEBUG) isCellInFieldPlanar(i,j);

				// prev column
				if (isCellInFieldPlanar(i-1,j-1))
				{
					if (ptrptrField[i-1][j-1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i,j-1))
				{
					if (ptrptrField[i][j-1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i+1,j-1))
				{
					if (ptrptrField[i+1][j-1] > 0) 
					{
						neighbours++;
					}
				}

				// column
				if (isCellInFieldPlanar(i-1,j))
				{
					if (ptrptrField[i-1][j] > 0)
					{
						neighbours++;
					}
				}
				// cell
				if (isCellInFieldPlanar(i+1,j))
				{
					if (ptrptrField[i+1][j] > 0)
					{
						neighbours++;
					}
				}

				// next column
				if (isCellInFieldPlanar(i-1,j+1))
				{
					if (ptrptrField[i-1][j+1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i,j+1))
				{
					if (ptrptrField[i][j+1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i+1,j+1))
				{
					if (ptrptrField[i+1][j+1] > 0)
					{
						neighbours++;
					}
				}
				if (DEBUG) printf("j=%d//i=%d  ",j,i);
				if (DEBUG) printf("%d // neighbours=%d", ptrptrTempField[i][j], neighbours);
				switch (neighbours)
				{
					// isolation
				case 0 :
				case 1 :
					if (ptrptrTempField[i][j] == 1)
					{
						gone++;
					}
					ptrptrTempField[i][j] = 0;
					break;
					// surviving cells
				case 2 : 
					break;
				case 3 :
					if (ptrptrTempField[i][j] == 1)
					{
					}
					else
					{
						ptrptrTempField[i][j] = 1;
						born++;
					}
					break;
					// overpopulation
				case 4 :
				case 5 : 
				case 6 : 
				case 7 :
				case 8 :
					if (ptrptrTempField[i][j] == 1)
					{
						gone++;
					}
					ptrptrTempField[i][j] = 0;
					break;
				default  :
					printf("ERROR : neighbours = default");
					break;
				}
				if(ptrptrField[i][j] == 1)
				{
					alive++;
				}

			}
		}
		if (DEBUG) 
		{
			printf("\nnew field is\n");
			//Test-output for initialisation of new Array
			for(i = 0; i < fieldSize; ++i)
			{
				for(j = 0; j < fieldSize; ++j)
				{
					printf("%d ", ptrptrTempField[j][i]);
					if (j == fieldSize - 1)
					{
						printf("\n");
					}
				}
			}
		}
		alive = alive + born - gone;
		dead = (fieldSize * fieldSize) - alive;
		ptrptrFieldToDelete = ptrptrField;
		ptrptrField = ptrptrTempField;
		free(ptrptrFieldToDelete);
		ptrptrFieldToDelete = NULL;
		ptrptrTempField = NULL;
		if (!DEBUG)
		{
			printf("alive=%d dead=%d born=%d gone=%d", alive, dead, born, gone);
		}
		return 1;
	}
	else
	{
		return 0;
	}
}
/*
int isCellInFieldTorus(int x, int y)
{
	if (x < 0)
	{
		// leave left, enter right
	}
	else
	{
		if (x >= 0 && x < fieldSize)
		{
			// in regular field
		}
		else
		{
			// leave right, enter left
		}
	}

	if (x >= 0 && x < fieldSize)
	{
		if (y >= 0 && y < fieldSize)
		{
			return 0;
		}
		else
		{
			if(
			
		}
	}
	else
	{
		if (y >= 0 && y < fieldSize)
		{
			if (DEBUG) printf("not x");
		}
		else
		{
			if (DEBUG) printf("not xy");
		}
	}
	return 0;
}

int algorithmTorus(int x, int y)
{
	int neighbours = 0;
	alive = 0;
	dead = 0;
	born = 0;
	gone = 0;

	if (fieldSize > 0)
	{
		createTempField();

		if (!DEBUG) printf("created TempField\n");

		for(j = 0; j < fieldSize; ++j)
		{
			neighbours = 0;

			for(i = 0; i < fieldSize; ++i)
			{
				neighbours = 0;
				// prev column
				if (isCellInFieldPlanar(i-1,j-1))
				{
					if (ptrptrField[i-1][j-1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i,j-1))
				{
					if (ptrptrField[i][j-1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i+1,j-1))
				{
					if (ptrptrField[i+1][j-1] > 0) 
					{
						neighbours++;
					}
				}

				// column
				if (isCellInFieldPlanar(i-1,j))
				{
					if (ptrptrField[i-1][j] > 0)
					{
						neighbours++;
					}
				}
				// cell
				if (isCellInFieldPlanar(i+1,j))
				{
					if (ptrptrField[i+1][j] > 0)
					{
						neighbours++;
					}
				}

				// next column
				if (isCellInFieldPlanar(i-1,j+1))
				{
					if (ptrptrField[i-1][j+1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i,j+1))
				{
					if (ptrptrField[i][j+1] > 0)
					{
						neighbours++;
					}
				}
				if (isCellInFieldPlanar(i+1,j+1))
				{
					if (ptrptrField[i+1][j+1] > 0)
					{
						neighbours++;
					}
				}
				if (DEBUG) printf("j=%d//i=%d  ",j,i);
				if (DEBUG) printf("%d // neighbours=%d", ptrptrTempField[i][j], neighbours);
				switch (neighbours)
				{
					// isolation
				case 0 :
				case 1 :
					if (ptrptrTempField[i][j] == 1)
					{
						gone++;
					}
					ptrptrTempField[i][j] = 0;
					break;
					// surviving cells
				case 2 : 
					break;
				case 3 :
					if (ptrptrTempField[i][j] == 1)
					{
					}
					else
					{
						ptrptrTempField[i][j] = 1;
						born++;
					}
					break;
					// overpopulation
				case 4 :
				case 5 : 
				case 6 : 
				case 7 :
				case 8 :
					if (ptrptrTempField[i][j] == 1)
					{
						gone++;
					}
					ptrptrTempField[i][j] = 0;
					break;
				default  :
					printf("ERROR : neighbours = default");
					break;
				}
				if(ptrptrField[i][j] == 1)
				{
					alive++;
				}

			}
		}
		if (DEBUG) 
		{
			printf("\nnew field is\n");
			//Test-output for initialisation of new Array
			for(i = 0; i < fieldSize; ++i)
			{
				for(j = 0; j < fieldSize; ++j)
				{
					printf("%d ", ptrptrTempField[j][i]);
					if (j == fieldSize - 1)
					{
						printf("\n");
					}
				}
			}
		}
		alive = alive + born - gone;
		dead = (fieldSize * fieldSize) - alive;
		ptrptrFieldToDelete = ptrptrField;
		ptrptrField = ptrptrTempField;
		free(ptrptrFieldToDelete);
		ptrptrFieldToDelete = NULL;
		ptrptrTempField = NULL;
		if (!DEBUG)
		{
			printf("alive=%d dead=%d born=%d gone=%d", alive, dead, born, gone);
		}
		return 1;
	}
	else
	{
		return 0;
	}

}
*/